%Comparing MCF and EDF-VD
%by Zhishan Guo
%2016.1

clc;
clear all;

TOTALTries = 1000; %number of sets for every conbination of UHI ULO, 
%in our paper we set this to 10,000, here is for a quick demo

ULOArray = [0.4,0.45,0.5,0.55, 0.6, 0.65, 0.7, 0.75, 0.8, 0.85, 0.9, 0.95];
SchedulableEDFVD = zeros(size(ULOArray,2),TOTALTries);
SchedulableEDFVD_new = zeros(size(ULOArray,2),TOTALTries);
SchedulableMCF = zeros(size(ULOArray,2),TOTALTries);
SchedulableMCF_new = zeros(size(ULOArray,2),TOTALTries);

UHIHI = zeros(size(ULOArray,2),TOTALTries);
ULOHI = zeros(size(ULOArray,2),TOTALTries);

for ULOpercent_i = 1:size(ULOArray,2)
    ULO = ULOArray(ULOpercent_i)
    for trynumber = 1: TOTALTries
        n= round(5+15*rand()); %task number
        if mod(trynumber,1000)==0
            trynumber
            ULO
        end

        HIprob = 0.5; % prob of being HI
        %task set generation
        test_sumUH = 1.1;
        count = 0;
        while count <100 && (test_sumUH>1 || flag_FAIL>0)
            n_H = binornd(n,HIprob); 
            while n_H>n-2 || n_H<2
                n_H = binornd(n,HIprob); 
            end
            [U_L,U_H,flag_FAIL] = MCTaskGenerator_WLOHI(ULO,n,n_H);
            count = count+1;
            test_sumUH = sum(U_H);
        end
        if sum(test_sumUH) > 1 %generator failed
            ULOHI(ULOpercent_i,trynumber) = -1;
            UHIHI(ULOpercent_i,trynumber) = -1;
            continue;
        end
        
        %Utilization calculation
        UHIHI(ULOpercent_i,trynumber) = sum(U_H(1:n_H));
        ULOHI(ULOpercent_i,trynumber) = sum(U_H((n_H+1):n));
        
        %Schedulablity easy check
        if sum(U_L((n_H+1):n))+sum(U_H(1:n_H)) <= 1
            SchedulableEDFVD(ULOpercent_i,trynumber) = 1;
            SchedulableEDFVD_new(ULOpercent_i,trynumber) = 1;
            SchedulableMCF(ULOpercent_i,trynumber) = 1;
            SchedulableMCF_new(ULOpercent_i,trynumber) = 1;
            continue;
        end
        
        % under new model, scale this set into original Vestal model
        U_L_new = U_L;
        U_H_new = U_H;
        for i=1:n
            U_L_new(i) = U_L(i)/(1-ULOHI(ULOpercent_i,trynumber));
        end
        for i=1:n_H
            U_H_new(i) = U_H(i)/(1-ULOHI(ULOpercent_i,trynumber));
        end
        for i=(n_H+1):n
            U_H_new(i) = 0;
        end
        
        % Schedulablity under new model: Check EDF-VD Schedulablity
        EDFVD_x = sum(U_L_new(1:n_H))/(1-sum(U_L_new((n_H+1):n)));
        if EDFVD_x*sum(U_L_new((n_H+1):n))+sum(U_H_new(1:n_H)) <= 1 && EDFVD_x<=1 && EDFVD_x>=0
            SchedulableEDFVD_new(ULOpercent_i,trynumber) = 1;
        end
        
        % Schedulablity under new model: Check MCF Schedulablity
        pho = max([sum(U_L_new),sum(U_H_new(1:n_H))]);
        x0 = U_L_new';
        if pho <= 1
            for ii=1:n_H
                x0(ii) = U_L_new(ii)*U_H_new(ii)/pho/(U_H_new(ii)/pho - U_H_new(ii) + U_L_new(ii));
            end
            if sum(x0) <=1
                SchedulableMCF_new(ULOpercent_i,trynumber) = 1;
            end
        end
            
        %under old model
        for i=(n_H+1):n
            U_H(i) = 0;
        end
        
        % Schedulablity under old model: Check EDF-VD Schedulablity
        EDFVD_x = sum(U_L(1:n_H))/(1-sum(U_L((n_H+1):n)));
        if EDFVD_x*sum(U_L((n_H+1):n))+sum(U_H(1:n_H)) <= 1 && EDFVD_x<=1 && EDFVD_x>=0
            SchedulableEDFVD(ULOpercent_i,trynumber) = 1;
        end
        
        pho = max([sum(U_L),sum(U_H(1:n_H))]);
        x0 = U_L';
        if pho <= 1
            for ii=1:n_H
                x0(ii) = U_L(ii)*U_H(ii)/pho/(U_H(ii)/pho - U_H(ii) + U_L(ii));
            end
            if sum(x0) <=1
                SchedulableMCF(ULOpercent_i,trynumber) = 1;
            end
        end
    end
end

Schedulable_General = zeros (2,size(ULOArray,2));
for i=1:size(ULOArray,2)
    %Schedulable_General(1,:) = sum(SchedulableEDFVD,2)'./TOTALTries;
    Schedulable_General(2,:) = sum(max(0,SchedulableEDFVD_new),2)'./TOTALTries;
    %Schedulable_General(2,:) = sum(SchedulableMCF,2)'./TOTALTries;
    Schedulable_General(1,:) = sum(max(0,SchedulableMCF_new),2)'./TOTALTries;
end

% MC-Fluid schedulability is separately computed, and takes much longer (hours or days)

figure
plot(ULOArray+0.025,Schedulable_General,'g--x'),
xlabel('Normalized Utilization Bound') % x-axis label
ylabel('Acceptance Ratio') % y-axis label
legend('MCF','EDF-VD');
    